import { LifiStep, Route, Token } from '@lifi/types';
import { TenderlyResponse } from './types';
/**
 * Predefined hook that decrypts calldata using EIP-1193 compliant wallet functions.
 * @param {string} walletAddress - The wallet address of the user that should decrypt the calldata.
 * @return {(encryptedData: string) => Promise<any>} A function that decrypts data using EIP-1193 compliant wallet functions.
 */
export declare const getEthereumDecryptionHook: (walletAddress: string) => (encryptedData: string) => Promise<string>;
/**
 * Predefined hook that get the public encryption key of a user using EIP-1193 compliant wallet functions.
 * @param {string} walletAddress - The wallet address of the user.
 * @return {(walletAddress: string) => () => Promise<any>} A function that return the public encryption key using EIP-1193 compliant wallet functions.
 */
export declare const getEthereumPublicKeyHook: (walletAddress: string) => () => Promise<string>;
/**
 * Returns a random number between min (inclusive) and max (inclusive)
 */
export declare const getRandomNumber: (min: number, max: number) => number;
export declare const isSameToken: (tokenA: Token, tokenB: Token) => boolean;
export declare const checkPackageUpdates: (packageName?: string, packageVersion?: string, disableCheck?: boolean) => Promise<void>;
/**
 * Converts a quote to Route
 * @param {LifiStep} step - Step returned from the quote endpoint.
 * @return {Route} - The route to be executed.
 * @throws {ValidationError} Throws a ValidationError if the step has missing values.
 */
export declare const convertQuoteToRoute: (step: LifiStep) => Route;
export declare const fetchTxErrorDetails: (txHash: string, chainId: number) => Promise<TenderlyResponse>;
