"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const request_1 = require("../request");
const typeguards_1 = require("../typeguards");
const errors_1 = require("../utils/errors");
const parseError_1 = require("../utils/parseError");
const ConfigService_1 = __importDefault(require("./ConfigService"));
/**
 * @deprecated We don't want to support this endpoint anymore in the future. /chains, /tools, /connections, and /tokens should be used instead
 */
const getPossibilities = async (requestConfig, options) => {
    if (!requestConfig) {
        requestConfig = {};
    }
    const config = ConfigService_1.default.getInstance().getConfig();
    // apply defaults
    if (requestConfig.bridges || config.defaultRouteOptions.bridges) {
        requestConfig.bridges =
            requestConfig.bridges || config.defaultRouteOptions.bridges;
    }
    if (requestConfig.exchanges || config.defaultRouteOptions.exchanges) {
        requestConfig.exchanges =
            requestConfig.exchanges || config.defaultRouteOptions.exchanges;
    }
    // send request
    try {
        const response = await (0, request_1.request)(`${config.apiUrl}/advanced/possibilities`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(requestConfig),
            signal: options?.signal,
        });
        return response;
    }
    catch (e) {
        throw await (0, parseError_1.parseBackendError)(e);
    }
};
const getToken = async (chain, token, options) => {
    if (!chain) {
        throw new errors_1.ValidationError('Required parameter "chain" is missing.');
    }
    if (!token) {
        throw new errors_1.ValidationError('Required parameter "token" is missing.');
    }
    const config = ConfigService_1.default.getInstance().getConfig();
    try {
        const response = await (0, request_1.request)(`${config.apiUrl}/token?${new URLSearchParams({
            chain,
            token,
        })}`, {
            signal: options?.signal,
        });
        return response;
    }
    catch (e) {
        throw await (0, parseError_1.parseBackendError)(e);
    }
};
const getQuote = async (requestConfig, options) => {
    const config = ConfigService_1.default.getInstance().getConfig();
    const requiredParameters = [
        'fromChain',
        'fromToken',
        'fromAddress',
        'fromAmount',
        'toChain',
        'toToken',
    ];
    requiredParameters.forEach((requiredParameter) => {
        if (!requestConfig[requiredParameter]) {
            throw new errors_1.ValidationError(`Required parameter "${requiredParameter}" is missing.`);
        }
    });
    // apply defaults
    requestConfig.order || (requestConfig.order = config.defaultRouteOptions.order);
    requestConfig.slippage || (requestConfig.slippage = config.defaultRouteOptions.slippage);
    requestConfig.integrator || (requestConfig.integrator = config.defaultRouteOptions.integrator);
    requestConfig.referrer || (requestConfig.referrer = config.defaultRouteOptions.referrer);
    requestConfig.fee || (requestConfig.fee = config.defaultRouteOptions.fee);
    requestConfig.allowBridges || (requestConfig.allowBridges = config.defaultRouteOptions.bridges?.allow);
    requestConfig.denyBridges || (requestConfig.denyBridges = config.defaultRouteOptions.bridges?.deny);
    requestConfig.preferBridges || (requestConfig.preferBridges = config.defaultRouteOptions.bridges?.prefer);
    requestConfig.allowExchanges || (requestConfig.allowExchanges = config.defaultRouteOptions.exchanges?.allow);
    requestConfig.denyExchanges || (requestConfig.denyExchanges = config.defaultRouteOptions.exchanges?.deny);
    requestConfig.preferExchanges || (requestConfig.preferExchanges = config.defaultRouteOptions.exchanges?.prefer);
    Object.keys(requestConfig).forEach((key) => !requestConfig[key] &&
        delete requestConfig[key]);
    try {
        const response = await (0, request_1.request)(`${config.apiUrl}/quote?${new URLSearchParams(requestConfig)}`, {
            signal: options?.signal,
        });
        return response;
    }
    catch (e) {
        throw await (0, parseError_1.parseBackendError)(e);
    }
};
const getContractCallQuote = async (requestConfig, options) => {
    const config = ConfigService_1.default.getInstance().getConfig();
    // validation
    const requiredParameters = [
        'fromChain',
        'fromToken',
        'fromAddress',
        'toChain',
        'toToken',
        'toAmount',
        'toContractAddress',
        'toContractCallData',
        'toContractGasLimit',
    ];
    requiredParameters.forEach((requiredParameter) => {
        if (!requestConfig[requiredParameter]) {
            throw new errors_1.ValidationError(`Required parameter "${requiredParameter}" is missing.`);
        }
    });
    // apply defaults
    // option.order is not used in this endpoint
    requestConfig.slippage || (requestConfig.slippage = config.defaultRouteOptions.slippage);
    requestConfig.integrator || (requestConfig.integrator = config.defaultRouteOptions.integrator);
    requestConfig.referrer || (requestConfig.referrer = config.defaultRouteOptions.referrer);
    requestConfig.fee || (requestConfig.fee = config.defaultRouteOptions.fee);
    requestConfig.allowBridges || (requestConfig.allowBridges = config.defaultRouteOptions.bridges?.allow);
    requestConfig.denyBridges || (requestConfig.denyBridges = config.defaultRouteOptions.bridges?.deny);
    requestConfig.preferBridges || (requestConfig.preferBridges = config.defaultRouteOptions.bridges?.prefer);
    requestConfig.allowExchanges || (requestConfig.allowExchanges = config.defaultRouteOptions.exchanges?.allow);
    requestConfig.denyExchanges || (requestConfig.denyExchanges = config.defaultRouteOptions.exchanges?.deny);
    requestConfig.preferExchanges || (requestConfig.preferExchanges = config.defaultRouteOptions.exchanges?.prefer);
    // send request
    try {
        const response = await (0, request_1.request)(`${config.apiUrl}/quote/contractCall`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(requestConfig),
            signal: options?.signal,
        });
        return response;
    }
    catch (e) {
        throw await (0, parseError_1.parseBackendError)(e);
    }
};
const getContractCallsQuote = async (requestConfig, options) => {
    const config = ConfigService_1.default.getInstance().getConfig();
    // validation
    const requiredParameters = [
        'fromChain',
        'fromToken',
        'fromAddress',
        'toChain',
        'toToken',
        'toAmount',
        'contractCalls',
    ];
    if (requestConfig.contractCalls.length === 0) {
        throw new errors_1.ValidationError(`Parameter "contractCalls" is empty.`);
    }
    requiredParameters.forEach((requiredParameter) => {
        if (!requestConfig[requiredParameter]) {
            throw new errors_1.ValidationError(`Required parameter "${requiredParameter}" is missing.`);
        }
    });
    // apply defaults
    // option.order is not used in this endpoint
    requestConfig.slippage || (requestConfig.slippage = config.defaultRouteOptions.slippage);
    requestConfig.integrator || (requestConfig.integrator = config.defaultRouteOptions.integrator);
    requestConfig.referrer || (requestConfig.referrer = config.defaultRouteOptions.referrer);
    requestConfig.allowBridges || (requestConfig.allowBridges = config.defaultRouteOptions.bridges?.allow);
    requestConfig.denyBridges || (requestConfig.denyBridges = config.defaultRouteOptions.bridges?.deny);
    requestConfig.preferBridges || (requestConfig.preferBridges = config.defaultRouteOptions.bridges?.prefer);
    requestConfig.allowExchanges || (requestConfig.allowExchanges = config.defaultRouteOptions.exchanges?.allow);
    requestConfig.denyExchanges || (requestConfig.denyExchanges = config.defaultRouteOptions.exchanges?.deny);
    requestConfig.preferExchanges || (requestConfig.preferExchanges = config.defaultRouteOptions.exchanges?.prefer);
    // send request
    try {
        const response = await (0, request_1.request)(`${config.apiUrl}/quote/contractCalls`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(requestConfig),
            signal: options?.signal,
        });
        return response;
    }
    catch (e) {
        throw await (0, parseError_1.parseBackendError)(e);
    }
};
const getStatus = async (requestConfig, options) => {
    if (!requestConfig.txHash) {
        throw new errors_1.ValidationError('Required parameter "txHash" is missing.');
    }
    const config = ConfigService_1.default.getInstance().getConfig();
    const queryParams = new URLSearchParams(requestConfig);
    try {
        const response = await (0, request_1.request)(`${config.apiUrl}/status?${queryParams}`, {
            signal: options?.signal,
        });
        return response;
    }
    catch (e) {
        throw await (0, parseError_1.parseBackendError)(e);
    }
};
const getChains = async (options) => {
    const config = ConfigService_1.default.getInstance().getConfig();
    try {
        const response = await (0, request_1.request)(`${config.apiUrl}/chains`, {
            signal: options?.signal,
        });
        return response.chains;
    }
    catch (e) {
        throw await (0, parseError_1.parseBackendError)(e);
    }
};
const getRoutes = async (requestConfig, options) => {
    if (!(0, typeguards_1.isRoutesRequest)(requestConfig)) {
        throw new errors_1.ValidationError('Invalid routes request.');
    }
    const config = ConfigService_1.default.getInstance().getConfig();
    // apply defaults
    requestConfig.options = {
        ...config.defaultRouteOptions,
        ...requestConfig.options,
    };
    // send request
    try {
        const response = await (0, request_1.request)(`${config.apiUrl}/advanced/routes`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(requestConfig),
            signal: options?.signal,
        });
        return response;
    }
    catch (e) {
        throw await (0, parseError_1.parseBackendError)(e);
    }
};
const getStepTransaction = async (step, options) => {
    if (!(0, typeguards_1.isStep)(step)) {
        // While the validation fails for some users we should not enforce it
        // eslint-disable-next-line no-console
        console.warn('SDK Validation: Invalid Step', step);
    }
    const config = ConfigService_1.default.getInstance().getConfig();
    try {
        const response = await (0, request_1.request)(`${config.apiUrl}/advanced/stepTransaction`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(step),
            signal: options?.signal,
        });
        return response;
    }
    catch (e) {
        throw await (0, parseError_1.parseBackendError)(e);
    }
};
const getTools = async (requestConfig, options) => {
    const config = ConfigService_1.default.getInstance().getConfig();
    if (requestConfig) {
        Object.keys(requestConfig).forEach((key) => !requestConfig[key] &&
            delete requestConfig[key]);
    }
    const response = await (0, request_1.request)(`${config.apiUrl}/tools?${new URLSearchParams(requestConfig)}`, {
        signal: options?.signal,
    });
    return response;
};
const getTokens = async (requestConfig, options) => {
    const config = ConfigService_1.default.getInstance().getConfig();
    if (requestConfig) {
        Object.keys(requestConfig).forEach((key) => !requestConfig[key] &&
            delete requestConfig[key]);
    }
    const response = await (0, request_1.request)(`${config.apiUrl}/tokens?${new URLSearchParams(requestConfig)}`, {
        signal: options?.signal,
    });
    return response;
};
const getGasRecommendation = async ({ chainId, fromChain, fromToken }, options) => {
    const config = ConfigService_1.default.getInstance().getConfig();
    if (!chainId) {
        throw new errors_1.ValidationError('Required parameter "chainId" is missing.');
    }
    const url = new URL(`${config.apiUrl}/gas/suggestion/${chainId}`);
    if (fromChain) {
        url.searchParams.append('fromChain', fromChain);
    }
    if (fromToken) {
        url.searchParams.append('fromToken', fromToken);
    }
    try {
        const response = await (0, request_1.request)(url, {
            signal: options?.signal,
        });
        return response;
    }
    catch (e) {
        throw await (0, parseError_1.parseBackendError)(e);
    }
};
const getAvailableConnections = async (connectionRequest) => {
    const config = ConfigService_1.default.getInstance().getConfig();
    const url = new URL(`${config.apiUrl}/connections`);
    const { fromChain, fromToken, toChain, toToken } = connectionRequest;
    if (fromChain) {
        url.searchParams.append('fromChain', fromChain);
    }
    if (fromToken) {
        url.searchParams.append('fromToken', fromToken);
    }
    if (toChain) {
        url.searchParams.append('toChain', toChain);
    }
    if (toToken) {
        url.searchParams.append('toToken', toToken);
    }
    const connectionRequestArrayParams = [
        'allowBridges',
        'denyBridges',
        'preferBridges',
        'allowExchanges',
        'denyExchanges',
        'preferExchanges',
    ];
    connectionRequestArrayParams.forEach((parameter) => {
        const connectionRequestArrayParam = connectionRequest[parameter];
        if (connectionRequestArrayParam?.length) {
            connectionRequestArrayParam?.forEach((value) => {
                url.searchParams.append(parameter, value);
            });
        }
    });
    try {
        const response = await (0, request_1.request)(url);
        return response;
    }
    catch (e) {
        throw await (0, parseError_1.parseBackendError)(e);
    }
};
const getTransactionHistory = async (walletAnalyticsRequest) => {
    const config = ConfigService_1.default.getInstance().getConfig();
    if (!walletAnalyticsRequest.fromTimestamp) {
        throw new errors_1.ValidationError('Required parameter "fromTimestamp" is missing.');
    }
    if (!walletAnalyticsRequest.toTimestamp) {
        throw new errors_1.ValidationError('Required parameter "toTimestamp" is missing.');
    }
    const url = new URL(`${config.apiUrl}/analytics/wallets/${walletAnalyticsRequest.walletAddress}`);
    url.searchParams.append('fromTimestamp', walletAnalyticsRequest.fromTimestamp.toString());
    url.searchParams.append('toTimestamp', walletAnalyticsRequest.toTimestamp.toString());
    const response = await (0, request_1.request)(url);
    return response;
};
exports.default = {
    getChains,
    getContractCallQuote,
    getContractCallsQuote,
    getGasRecommendation,
    getPossibilities,
    getQuote,
    getRoutes,
    getStatus,
    getStepTransaction,
    getToken,
    getTokens,
    getTools,
    getAvailableConnections,
    getTransactionHistory,
};
