"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("../utils/errors");
const ApiService_1 = __importDefault(require("./ApiService"));
class ChainsService {
    constructor() {
        this.chains = [];
        this.loadingPromise = this.loadAvailableChains();
    }
    async loadAvailableChains() {
        try {
            this.chains = await ApiService_1.default.getChains();
        }
        catch (error) {
            // We try to load chains during initialization of the LiFi class and
            // because we no longer in scope of the constructor we fail silently here
        }
    }
    async checkLoading() {
        if (this.loadingPromise) {
            await this.loadingPromise;
        }
        if (!this.chains.length) {
            await this.loadAvailableChains();
        }
    }
    static getInstance() {
        if (!this.instance) {
            this.instance = new ChainsService();
        }
        return this.instance;
    }
    async getChainById(chainId) {
        await this.checkLoading();
        const chain = this.chains.find((chain) => chain.id === chainId);
        if (!chain) {
            throw new errors_1.ValidationError(`Unknown chainId passed: ${chainId}.`);
        }
        return chain;
    }
    async getChains() {
        await this.checkLoading();
        return this.chains;
    }
}
exports.default = ChainsService;
