"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handlePreRestart = void 0;
/* eslint-disable @typescript-eslint/no-non-null-assertion */
const ethers_1 = require("ethers");
const errors_1 = require("./errors");
const handlePreRestart = async (route, signer) => {
    for (let index = 0; index < route.steps.length; index++) {
        const stepHasFailed = route.steps[index].execution?.status === 'FAILED';
        if (stepHasFailed) {
            await handleErrorType(route, index, signer);
            deleteFailedProcesses(route, index);
            deleteTransactionData(route, index);
        }
    }
};
exports.handlePreRestart = handlePreRestart;
const handleErrorType = async (route, index, signer) => {
    const isGasLimitError = route.steps[index].execution?.process.some((p) => p.error?.code === errors_1.LifiErrorCode.GasLimitError);
    const isGasPriceError = route.steps[index].execution?.process.some((p) => p.error?.code === errors_1.LifiErrorCode.TransactionUnderpriced);
    const { transactionRequest } = route.steps[index];
    if (isGasLimitError) {
        if (transactionRequest) {
            let gasLimit = transactionRequest.gasLimit;
            try {
                gasLimit = await signer.estimateGas(transactionRequest);
            }
            catch (error) { }
            if (gasLimit) {
                transactionRequest.gasLimit = ethers_1.BigNumber.from(`${(BigInt(gasLimit.toString()) * 125n) / 100n}`);
            }
        }
        route.steps[index].estimate.gasCosts?.forEach((gasCost) => (gasCost.limit = `${Math.round(Number(gasCost.limit) * 1.25)}`));
    }
    if (isGasPriceError) {
        if (transactionRequest) {
            let gasPrice = transactionRequest.gasPrice;
            try {
                gasPrice = await signer.getGasPrice();
            }
            catch (error) { }
            if (gasPrice) {
                transactionRequest.gasPrice = ethers_1.BigNumber.from(`${(BigInt(gasPrice.toString()) * 125n) / 100n}`);
            }
        }
        route.steps[index].estimate.gasCosts?.forEach((gasCost) => (gasCost.price = `${Math.round(Number(gasCost.price) * 1.25)}`));
    }
};
const deleteFailedProcesses = (route, index) => {
    if (route.steps[index].execution) {
        route.steps[index].execution.process = route.steps[index].execution.process.filter((process) => process.status === 'DONE');
    }
};
const deleteTransactionData = (route, index) => {
    route.steps[index].transactionRequest = undefined;
};
